import csv
from io import StringIO


class BinaryIterator(object):
    def __init__(self, content):
        self.content = content

    def __iter__(self):
        yield self.content
        raise StopIteration



import os

from akagi.log import logger


class DataSource(object):
    '''DataSource is an base class of all data sources
    '''

    def save(self, tar_dir):
        for d in self.bundle.data_files:

            path = os.path.expanduser(os.path.join(tar_dir, d.key))
            dirname = os.path.dirname(path)

            if not os.path.isdir(dirname):
                os.makedirs(dirname)

            logger.info("Saved %(key)s to %(path)s" % ({"key": d.key, "path": path}))

            with open(path, 'wb') as f:
                f.write(d.content)

    def __enter__(self):
        return self

    def __exit__(self, *exc):
        raise NotImplementedError



from akagi.iterators import CSVIterator, BinaryIterator

from enum import Enum


class FileFormat(Enum):
    CSV = 1
    BINARY = 2


class Iterator(object):

    @classmethod
    def get_iterator_class(cls, file_format):
        if file_format in [FileFormat.CSV, 'csv']:
            return CSVIterator
        elif file_format in [FileFormat.BINARY, 'binary']:
            return BinaryIterator
        else:
            raise Exception("Unsupported file format %(file_format)s." % locals())



import csv
from io import StringIO


class CSVIterator(object):
    def __init__(self, content):
        self.content = content

    def __iter__(self):
        return csv.reader(StringIO(self.content.decode('utf-8')))



import gzip
import io

from akagi.data_file import DataFile


class S3DataFile(DataFile):
    def __init__(self, obj, iterator_class):
        self.obj = obj
        self.iterator_class = iterator_class

    @property
    def filename(self):
        return self.key

    @property
    def key(self):
        return self.obj.key

    @property
    def content(self):
        if self._is_gzip():
            return gzip.decompress(self.obj.get()["Body"].read())
        else:
            return self.obj.get()["Body"].read()

    def __iter__(self):
        return iter(self.iterator_class(self.content))
